/*--------------------------------------------------------------------------
  W7100.H
 Registers definition for W7100
-------------------------------------------------------------------------- */

#ifndef _W7100_H_
#define _W7100_H_
#ifdef __IAR_SYSTEMS_ICC__
#ifndef _SYSTEM_BUILD
   #pragma system_include
#endif
#pragma language=extended


/*-------------------------------------------------------------------------
 *   W7100 Core
 *-------------------------------------------------------------------------*/

__sfr __no_init volatile union
{
  unsigned char ACC; /* Accumulator */
  struct /* Accumulator */
  {
    unsigned char ACC0 : 1;
    unsigned char ACC1 : 1;
    unsigned char ACC2 : 1;
    unsigned char ACC3 : 1;
    unsigned char ACC4 : 1;
    unsigned char ACC5 : 1;
    unsigned char ACC6 : 1;
    unsigned char ACC7 : 1;
  } ACC_bit;
} @ 0xE0;
__sfr __no_init volatile union
{
  unsigned char PSW; /* Program Status Word */
  struct /* Program Status Word */
  {
    unsigned char P : 1;
    unsigned char F1 : 1;
    unsigned char OV : 1;
    unsigned char RS0 : 1;
    unsigned char RS1 : 1;
    unsigned char F0 : 1;
    unsigned char AC : 1;
    unsigned char CY : 1;
  } PSW_bit;
} @ 0xD0;
__sfr __no_init volatile unsigned char SP @ 0x81; /* Stack Pointer */
__sfr __no_init volatile unsigned int DPTR0 @ 0x82; /* Data Pointer0 */
__sfr __no_init volatile unsigned char DPL @ 0x82; /* Data Pointer0 Low byte, LSB of DPTR */
__sfr __no_init volatile unsigned char DPL0 @ 0x82; /* Data Pointer0 Low byte, LSB of DPTR */
__sfr __no_init volatile unsigned char DPH @ 0x83; /* Data Pointer0 High byte, MSB of DPTR */
__sfr __no_init volatile unsigned char DPH0 @ 0x83; /* Data Pointer0 High byte, MSB of DPTR */
__sfr __no_init volatile unsigned int DPTR1 @ 0x84; /* Data Pointer1 */
__sfr __no_init volatile unsigned char DPL1 @ 0x84; /* Data Pointer1 Low byte, LSB of DPTR */
__sfr __no_init volatile unsigned char DPH1 @ 0x85; /* Data Pointer1 High byte, MSB of DPTR */
__sfr __no_init volatile unsigned char DPS @ 0x86; /* Data Pointer Selector */
__sfr __no_init volatile union
{
  unsigned char B; /* B Register */
  struct /* B Register */
  {
    unsigned char B0 : 1;
    unsigned char B1 : 1;
    unsigned char B2 : 1;
    unsigned char B3 : 1;
    unsigned char B4 : 1;
    unsigned char B5 : 1;
    unsigned char B6 : 1;
    unsigned char B7 : 1;
  } B_bit;
} @ 0xF0;
__sfr __no_init volatile unsigned char PCON @ 0x87; /* Power Control */
__sfr __no_init volatile unsigned char CKCON @ 0x8E; /* XDATA Wait States */
__sfr __no_init volatile unsigned char WTST @ 0x92; /* Program Wait States */
__sfr __no_init volatile unsigned char DPX0 @ 0x93; /* Data Page Pointer 0 */
__sfr __no_init volatile unsigned char DPX1 @ 0x95; /* Data Page Pointer 1 */
__sfr __no_init volatile unsigned char WDCON @ 0xD8; 
__sfr __no_init volatile unsigned char MXAX @ 0xEA; /* MOVX @Ri High address     */
__sfr __no_init volatile unsigned char ISPID @ 0xF1; /* ID for ISP*/
__sfr __no_init volatile unsigned int ISPADDR16 @ 0xF2; /* 16bit Address for ISP */
__sfr __no_init volatile unsigned char ISPDATA @ 0xF4; 
__sfr __no_init volatile unsigned char CKCBK @ 0xF5;  /* CKCON backup register */
__sfr __no_init volatile unsigned char DPX0BK @ 0xF6;  /* DPX0 backup Register   */
__sfr __no_init volatile unsigned char DPX1BK @ 0xF7;  /* DPX1 backup Register   */
__sfr __no_init volatile unsigned char DPSBK @ 0xF9; 
__sfr __no_init volatile unsigned int RAMBA16 @ 0xFA;
__sfr __no_init volatile unsigned char TMPR0 @ 0xFA; 
__sfr __no_init volatile unsigned char TMPR1 @ 0xFB; 
__sfr __no_init volatile unsigned int RAMEA16 @ 0xFC;
__sfr __no_init volatile unsigned char TMPR2 @ 0xFC; 
__sfr __no_init volatile unsigned char TMPR3 @ 0xFD;
__sfr __no_init volatile unsigned char WCONF @ 0xFF; /* W7100 Config Register */
                          /* 7    6    5   4   3    2   1  0  */
                          /* RB ISPEN EM2 EM1 EM0 F64EN FB BS */
                          /* 
                             RB      : 1 - Reboot after ISP done(APP Entry(0xFFF7~0xFFFF) Enable), 0 - No reboot
                             ISPEN   : 0 - Enable ISP in Boot built in W7100 , 1 - Disable
                             EM[2:0] : External Memory Mode
                             F64EN   : 0 - Boot Mode, 1 - Flash 64Kbyte Enable
                             FB      : Flash Busy Flag for ISP
                             BS      : Boot Selection (0 - Apps Running, 1 - Boot Running)
                           */    

/*-------------------------------------------------------------------------
 *   Timers
 *-------------------------------------------------------------------------*/


__sfr __no_init volatile union
{
  unsigned char TCON; /* Timer Control */
  struct /* Timer Control */
  {
    unsigned char IT0 : 1;
    unsigned char IE0 : 1;
    unsigned char IT1 : 1;
    unsigned char IE1 : 1;
    unsigned char TR0 : 1;
    unsigned char TF0 : 1;
    unsigned char TR1 : 1;
    unsigned char TF1 : 1;
  } TCON_bit;
} @ 0x88;
__sfr __no_init volatile unsigned char TMOD @ 0x89; /* Timer 0, 1 MODE */
__sfr __no_init volatile unsigned char TL0 @ 0x8A; /* Timer0 Low */
__sfr __no_init volatile unsigned char TL1 @ 0x8B; /* Timer1 Low */
__sfr __no_init volatile unsigned char TH0 @ 0x8C; /* Timer0 High */
__sfr __no_init volatile unsigned char TH1 @ 0x8D; /* Timer1 High */  
__sfr __no_init volatile unsigned char TA @ 0xC7; /* Timed Access Register */
__sfr __no_init volatile unsigned char T2CON @ 0xC8; /* Timer2 Control */
__sfr __no_init volatile unsigned char RLDL @ 0xCA; /* Capture Register of Timer2, Low */
__sfr __no_init volatile unsigned char RLDH @ 0xCB; /* Capture Register of Timer2, High */
__sfr __no_init volatile unsigned char TL2 @ 0xCC; /* Timer2 Low */
__sfr __no_init volatile unsigned char TH2 @ 0xCD; /* Timer2 High */  

/*-------------------------------------------------------------------------
 *   Interrupt
 *-------------------------------------------------------------------------*/

__sfr __no_init volatile union
{
  unsigned char IP; /* Interrupt Priority */
  struct /* Interrupt Priority */
  {
    unsigned char PX0 : 1;
    unsigned char PT0 : 1;
    unsigned char PX1 : 1;
    unsigned char PT1 : 1;
    unsigned char PS : 1;
    unsigned char  : 1;
    unsigned char  : 1;
    unsigned char  : 1;
  } IP_bit;
} @ 0xB8;

__sfr __no_init volatile union
{
  unsigned char IE; /* Interrupt Enable */
  struct /* Interrupt Enable */
  {
    unsigned char EX0 : 1;
    unsigned char ET0 : 1;
    unsigned char EX1 : 1;
    unsigned char ET1 : 1;
    unsigned char ES : 1;
    unsigned char ET2 : 1;
    unsigned char  : 1;
    unsigned char EA : 1;
  }IE_bit;
}@0xA8;

#if 0
__sfr __no_init volatile union
{
  unsigned char EIE; /* External Interrupt Enable */
  struct /* Interrupt Enable */
  {
    unsigned char EINT2 : 1;
    unsigned char EINT3 : 1;
    unsigned char  : 1;
    unsigned char EINT5 : 1;
    unsigned char EWDI : 1;
    unsigned char  : 1;
    unsigned char  : 1;
    unsigned char  : 1;
  } EIE_bit;
} @ 0xE8;

	
__sfr __no_init volatile union
{
  unsigned char EIP; /* External Interrupt Priority */
  struct /* Interrupt Enable */
  {
    unsigned char PINT2 : 1;
    unsigned char PINT3 : 1;
    unsigned char  : 1;
    unsigned char PINT5 : 1;
    unsigned char PWDI : 1;
    unsigned char  : 1;
    unsigned char  : 1;
    unsigned char  : 1;
  } EIP_bit;
} @ 0xF8;
#endif

__sfr __no_init volatile unsigned char EIF @ 0x91; /* Extended Interrupt Flag   */

/*-------------------------------------------------------------------------
 *   I/O Port
 *-------------------------------------------------------------------------*/

__sfr __no_init volatile union
{
  unsigned char P0; /* Port 0 */
  struct /* Port 0 */
  {
    unsigned char P0_0 : 1;
    unsigned char P0_1 : 1;
    unsigned char P0_2 : 1;
    unsigned char P0_3 : 1;
    unsigned char P0_4 : 1;
    unsigned char P0_5 : 1;
    unsigned char P0_6 : 1;
    unsigned char P0_7 : 1;
  } P0_bit;
} @ 0x80;
__sfr __no_init volatile union
{
  unsigned char P1; /* Port 1 */
  struct /* Port 1 */
  {
    unsigned char P1_0 : 1;
    unsigned char P1_1 : 1;
    unsigned char P1_2 : 1;
    unsigned char P1_3 : 1;
    unsigned char P1_4 : 1;
    unsigned char P1_5 : 1;
    unsigned char P1_6 : 1;
    unsigned char P1_7 : 1;
  } P1_bit;
} @ 0x90;
__sfr __no_init volatile union
{
  unsigned char P2; /* Port 2 */
  struct /* Port 2 */
  {
    unsigned char P2_0 : 1;
    unsigned char P2_1 : 1;
    unsigned char P2_2 : 1;
    unsigned char P2_3 : 1;
    unsigned char P2_4 : 1;
    unsigned char P2_5 : 1;
    unsigned char P2_6 : 1;
    unsigned char P2_7 : 1;
  } P2_bit;
} @ 0xA0;
__sfr __no_init volatile union
{
  unsigned char P3; /* Port 3 */
  struct /* Port 3 */
  {
    unsigned char P3_0 : 1;
    unsigned char P3_1 : 1;
    unsigned char P3_2 : 1;
    unsigned char P3_3 : 1;
    unsigned char P3_4 : 1;
    unsigned char P3_5 : 1;
    unsigned char P3_6 : 1;
    unsigned char P3_7 : 1;
  } P3_bit;
} @ 0xB0;

/*-------------------------------------------------------------------------
 *   Serial I/O Port
 *-------------------------------------------------------------------------*/

__sfr __no_init volatile unsigned char SBUF @ 0x99; /* Serial Data Buffer */
__sfr __no_init volatile union
{
  unsigned char SCON; /* Serial Control */
  struct /* Serial Control */
  {
    unsigned char RI : 1;
    unsigned char TI : 1;
    unsigned char RB8 : 1;
    unsigned char TB8 : 1;
    unsigned char REN : 1;
    unsigned char SM2 : 1;
    unsigned char SM1 : 1;
    unsigned char SM0 : 1;
  } SCON_bit;
} @ 0x98;

/* Extended  Interrupt Enable */
__sfr __no_init volatile union
{
  unsigned char EIE; 
  struct 
  {
    unsigned char EINT2 : 1;
    unsigned char EINT3 : 1;
    unsigned char EINT4 : 1;
    unsigned char EINT5 : 1;
    unsigned char EWDI : 1;
    unsigned char  : 1;
    unsigned char  : 1;
    unsigned char  : 1;
  } EIE_bit;
} @ 0xE8;

/* Extended  Interrupt Priority */
__sfr __no_init volatile union
{
  unsigned char EIP; 
  struct 
  {
    unsigned char PINT2 : 1;
    unsigned char PINT3 : 1;
    unsigned char PINT4 : 1;
    unsigned char PINT5 : 1;
    unsigned char PWDI : 1;
    unsigned char  : 1;
    unsigned char  : 1;
    unsigned char  : 1;
  } EIP_bit;
} @ 0xF8;
/*-------------------------------------------------------------------------
  BIT Values  
  -------------------------------------------------------------------------*/

/* TMOD Bit Values */
  #define T0_M0_	0x01
  #define T0_M1_   	0x02
  #define T0_CT_   	0x04
  #define T0_GATE_ 	0x08
  #define T1_M0_   	0x10
  #define T1_M1_   	0x20
  #define T1_CT_   	0x40
  #define T1_GATE_ 	0x80

/* CKCON Bit Values  */
  #define MD_    	0x07
  #define T0M_   	0x08
  #define T1M_   	0x10
  #define T2M_   	0x20


/*
 * Interrupt Vectors
 */
#define extern0 0x03 /* External interrupt 0 */
#define IE0_int 0x03 /* External interrupt 0 */
#define TF0_int 0x0B /* Timer 0 Interrupt */
#define timer0 0x0B /* Timer 0 Interrupt */
#define IE1_int 0x13 /* External interrupt 1 */
#define extern1 0x13 /* External interrupt 1 */
#define TF1_int 0x1B /* Timer 1 Interrupt */
#define timer1 0x1B /* Timer 1 Interrupt */
#define TI_int 0x23 /* Serial Port Interrupt */
#define SerialTI 0x23 /* Serial Port Interrupt */
#define RI_int 0x23 /* Serial Port Interrupt */
#define SerialRE 0x23 /* Serial Port Interrupt */

#define W7100_VECTOR  0x5B // should be changed

#pragma language=default
#endif  /* __IAR_SYSTEMS_ICC__  */

/***************************************************************************
 *   Assembler definitions
 *
 *   The following SFRs are built in in the a8051.exe and can not be
 *   defined explicitly in this file:
 *     ACC,B,PSW,SP,DPL,DPH
 *   The PSW-bits are built-in in the a8051.exe
 *     OV,AC,F0,RS0,RS1,P
 *
 ***************************************************************************/

#ifdef __IAR_SYSTEMS_ASM__


/*-------------------------------------------------------------------------
 *   Timers
 *-------------------------------------------------------------------------*/

TCON DEFINE 0x88 /* Timer Control */
TCON_IT0 DEFINE 0x88.0
TCON_IE0 DEFINE 0x88.1
TCON_IT1 DEFINE 0x88.2
TCON_IE1 DEFINE 0x88.3
TCON_TR0 DEFINE 0x88.4
TCON_TF0 DEFINE 0x88.5
TCON_TR1 DEFINE 0x88.6
TCON_TF1 DEFINE 0x88.7
TMOD DEFINE 0x89 /* Timer Mode */
TL0 DEFINE 0x8A /* Timer0 Low */
TL1 DEFINE 0x8B /* Timer1 Low */
TH0 DEFINE 0x8C /* Timer0 High */
TH1 DEFINE 0x8D /* Timer1 High */

/*-------------------------------------------------------------------------
 *   Interrupt
 *-------------------------------------------------------------------------*/

IE DEFINE 0xA8 /* Interrupt Enable */
IE_EX0 DEFINE 0xA8.0
IE_ET0 DEFINE 0xA8.1
IE_EX1 DEFINE 0xA8.2
IE_ET1 DEFINE 0xA8.3
IE_ES DEFINE 0xA8.4
IE_ET2 DEFINE 0xA8.5
IE_EA DEFINE 0xA8.7
IP DEFINE 0xB8 /* Interrupt Priority */
IP_PX0 DEFINE 0xB8.0
IP_PT0 DEFINE 0xB8.1
IP_PX1 DEFINE 0xB8.2
IP_PT1 DEFINE 0xB8.3
IP_PS DEFINE 0xB8.4
IP_PT2 DEFINE 0xB8.5

/*-------------------------------------------------------------------------
 *   I/O Port
 *-------------------------------------------------------------------------*/

P0 DEFINE 0x80 /* Port 0 */
P0_P00 DEFINE 0x80.0
P0_P01 DEFINE 0x80.1
P0_P02 DEFINE 0x80.2
P0_P03 DEFINE 0x80.3
P0_P04 DEFINE 0x80.4
P0_P05 DEFINE 0x80.5
P0_P06 DEFINE 0x80.6
P0_P07 DEFINE 0x80.7
P1 DEFINE 0x90 /* Port 1 */
P1_T2 DEFINE 0x90.0
P1_T2EX DEFINE 0x90.1
P2 DEFINE 0xA0 /* Port 2 */
P2_AD8 DEFINE 0xA0.0
P2_AD9 DEFINE 0xA0.1
P2_AD10 DEFINE 0xA0.2
P2_AD11 DEFINE 0xA0.3
P2_AD12 DEFINE 0xA0.4
P2_AD13 DEFINE 0xA0.5
P2_AD14 DEFINE 0xA0.6
P2_AD15 DEFINE 0xA0.7
P3 DEFINE 0xB0 /* Port 3 */
P3_RxD DEFINE 0xB0.0
P3_TxD DEFINE 0xB0.1
P3_INT0 DEFINE 0xB0.2
P3_INT1 DEFINE 0xB0.3
P3_T0 DEFINE 0xB0.4
P3_T1 DEFINE 0xB0.5
P3_WR DEFINE 0xB0.6
P3_RD DEFINE 0xB0.7

/*-------------------------------------------------------------------------
 *   Serial I/O Port
 *-------------------------------------------------------------------------*/

SCON DEFINE 0x98 /* Serial Control */
SCON_RI DEFINE 0x98.0
SCON_TI DEFINE 0x98.1
SCON_RB8 DEFINE 0x98.2
SCON_TB8 DEFINE 0x98.3
SCON_REN DEFINE 0x98.4
SCON_SM2 DEFINE 0x98.5
SCON_SM1 DEFINE 0x98.6
SCON_SM0 DEFINE 0x98.7
SBUF DEFINE 0x99 /* Serial Data Buffer */

/*-------------------------------------------------------------------------
 *   System Management
 *-------------------------------------------------------------------------*/

PCON DEFINE 0x87 /* Power Control */

#endif /* __IAR_SYSTEMS_ASM__*/
#endif /*_W7100_H_*/
