#include <stdio.h>			// to use printf
#include "serial.h" 		// serial related functions
#include "socket.h" 		// W7100A driver file
#include "types.h"
#include "W7100.h"
#include "Wizmemcpy.h"
#include "TCPIPcore.h"
#include "delay.h"
#include "loopback.h"

#define	MAX_BUF_SIZE	8096		// Maximum receive buffer size
uint8 __xdata * data_buf =(uint8 __xdata *)0x7000;	// Position of receive buffer
void Init_iMCU(); 		
void Init_Network(); 	

void main()
{

  Init_iMCU();		// Initialize 8051
  Init_Network();


  printf("\r\n====================");
  printf("\r\nW7100 using IAR_");
  printf("\r\n====================");

  printf("\r\nTest");

  //start message
  printf("\r\n----START----------\r\n");

  while(1){
	loopback_tcps(0,5000,(uint8 *)data_buf,0);
        loopback_tcps(1,5000,(uint8 *)data_buf,0);
        loopback_tcps(2,5000,(uint8 *)data_buf,0);
        loopback_tcps(3,5000,(uint8 *)data_buf,0);
        loopback_tcps(4,5000,(uint8 *)data_buf,0);
        loopback_tcps(5,5000,(uint8 *)data_buf,0);
        loopback_tcps(6,5000,(uint8 *)data_buf,0);
        loopback_tcps(7,5000,(uint8 *)data_buf,0);
  }



}

void Init_iMCU(void)
{
	uint8  i;

	IE_bit.EA = 0; 		// Disable all interrupts
	CKCON = 0x02;		// External Memory Access Time
	WTST = 0x03;
	WCONF = 0x40;
	EIP_bit.PINT5 = 1;
	IINCHIP_WRITE((uint16)IMR,0xff);
	for(i=0;i<MAX_SOCK_NUM;i++) IINCHIP_WRITE(Sn_IMR(i),0x1f);
	IINCHIP_ISR_ENABLE();
	

	InitSerial(); 	// Initialize serial port (Refer to serial.c)

	IE_bit.EA = 1;

}

   uint8 __xdata str[17];
void Init_Network(void)
{


    uint8  ip[4] = {192,168,1,123};                   // for setting SIP register
    uint8  gw[4] = {192,168,1,1};                     // for setting GAR register
    uint8  sn[4] = {255,255,255,0};                     // for setting SUBR register
    uint8  mac[6] = {0x00,0x08,0xDC,0x11,0x22,0x34};      // for setting SHAR register
    uint8  txsize[MAX_SOCK_NUM] = {8,2,1,1,1,1,1,1};
    uint8  rxsize[MAX_SOCK_NUM] = {8,2,1,1,1,1,1,1};


    /* Write MAC address of W7100 to SHAR register */
    IINCHIP_WRITE ((uint16 )SIPR0+0,ip[0]);
    IINCHIP_WRITE ((uint16 )SIPR0+1,ip[1]);
    IINCHIP_WRITE ((uint16 )SIPR0+2,ip[2]);
    IINCHIP_WRITE ((uint16 )SIPR0+3,ip[3]);

    /* Write GATEWAY of W7100 to GAR register */
    IINCHIP_WRITE ((uint16 )GAR0+0, gw[0]);
    IINCHIP_WRITE ((uint16 )GAR0+1, gw[1]);
    IINCHIP_WRITE ((uint16 )GAR0+2, gw[2]);
    IINCHIP_WRITE ((uint16 )GAR0+3, gw[3]);

    /* Write SUBNETMASK of W7100 to SURR register */
    IINCHIP_WRITE ((uint16 )SUBR0+0, sn[0]);
    IINCHIP_WRITE ((uint16 )SUBR0+1, sn[1]);
    IINCHIP_WRITE ((uint16 )SUBR0+2, sn[2]);
    IINCHIP_WRITE ((uint16 )SUBR0+3, sn[3]);

    /* Write MAC address of W7100 to SHAR register */
    IINCHIP_WRITE ((uint16 )SHAR0+0, mac[0]);
    IINCHIP_WRITE ((uint16 )SHAR0+1, mac[1]);
    IINCHIP_WRITE ((uint16 )SHAR0+2, mac[2]);
    IINCHIP_WRITE ((uint16 )SHAR0+3, mac[3]);
    IINCHIP_WRITE ((uint16 )SHAR0+4, mac[4]);
    IINCHIP_WRITE ((uint16 )SHAR0+5, mac[5]);

    set_MEMsize((uint8 *)txsize,(uint8 *)rxsize);

    while(!(EIF & 0x02));
    EIF &= ~0x02;

    printf( "==============================================\r\n");
    printf( "       W7100   Net Config Information         \r\n");
    printf( "==============================================\r\n");

    printf( "MAC ADDRESS IP : %.2x.%.2x.%.2x.%.2x.%.2x.%.2x\r\n",
            IINCHIP_READ ((uint16 )SHAR0+0), IINCHIP_READ ((uint16 )SHAR0+1),
            IINCHIP_READ ((uint16 )SHAR0+2), IINCHIP_READ ((uint16 )SHAR0+3),
            IINCHIP_READ ((uint16 )SHAR0+4), IINCHIP_READ ((uint16 )SHAR0+5)
            ) ;
    sprintf(str,"%.3d.%.3d.%.3d.%.3d",
           IINCHIP_READ ((uint16 )SUBR0+0), IINCHIP_READ ((uint16 )SUBR0+1),
           IINCHIP_READ ((uint16 )SUBR0+2), IINCHIP_READ ((uint16 )SUBR0+3)
            );
    printf( "SUBNET MASK  : %s\r\n",str);

    sprintf(str,"%.3d.%.3d.%.3d.%.3d",
          IINCHIP_READ ((uint16 )GAR0+0), IINCHIP_READ ((uint16 )GAR0+1),
          IINCHIP_READ ((uint16 )GAR0+2), IINCHIP_READ ((uint16 )GAR0+3)
            );
    printf( "G/W IP ADDRESS : %s\r\n",str);

    sprintf(str,"%.3d.%.3d.%.3d.%.3d",
                    IINCHIP_READ ((uint16 )SIPR0+0), IINCHIP_READ ((uint16 )SIPR0+1),
                    IINCHIP_READ ((uint16 )SIPR0+2), IINCHIP_READ ((uint16 )SIPR0+3));

    printf( "LOCAL IP ADDRESS : %s\r\n", str);

}

