#include "W7100.h"
#include "serial.h"
void InitSerial(void)
{
	IE_bit.ET1 = 0;		/* TIMER1 INT DISABLE */
	TMOD = 0x20;  	/* TIMER MODE 2 */
	PCON |= 0x80;		/* SMOD = 1 */

//	TH1 = 0xFD;		/* X2 19200(SMOD=1) at 11.0592 MHZ */
//	TH1 = 0xE8;		/* X2 19200(SMOD=1) at 88.4736 MHz */
//	TH1 = 0xF8;		/* X2 57600(SMOD=1) at 88.4736 MHz */
	TH1 = 0xFC;		/* X2 115200(SMOD=1) at 88.4736 MHz */
//	TH1 = 0xFE;		/* X2 115200(SMOD=1) at 44.2368 MHz */

	TCON_bit.TR1 = 1;		/* START THE TIMER1 */	
	SCON = 0x52;		/* SERIAL MODE 1, REN=1, TI=1, RI=0 */

	/* Interrupt */
	IE_bit.ES   = 0;	 	/* Serial interrupt disable */
	SCON_bit.RI   = 0; 		
	SCON_bit.TI   = 0; 		
}

int putchar(int outchar)
{

	// Write data into serial-buffer.
	SBUF = outchar;
	// Wait till data recording is finished.
	while(!SCON_bit.TI);
	SCON_bit.TI = 0;
	
	return outchar;

}

#if 0
//#ifndef __SERIAL_UNUSED
/*
Description   :  Output 1 character through Serial Port
Argument      :  byData - character to output(INPUT)
Return Value  :
Note          :
*/
void PutByte(unsigned char byData)
{
	// Write data into serial-buffer.
	SBUF = byData;
	// Wait till data recording is finished.
	while(!SCON_bit.TI);
	SCON_bit.TI = 0;
}

/*
Description   :  Read 1 character from Serial.
Argument      :
Return Value  :   Read 1 character from Serial and Return.
Note          :
*/
unsigned char GetByte(void)	
{
	unsigned char byData;
	// Wait till data is received.
	while(!SCON_bit.RI);		
	SCON_bit.RI = 0;
	// Read data.
	byData = SBUF;		
	return byData;
}


/*
Description   :  Check to input to Serial or not.
Argument      :
Return Value  :    1)If there's input, then returned value is '1'.
	           2)If there's no input, then returned value is '-1'.
Note          :
*/
char IsPressedKey()
{
	if( SCON_bit.RI == 1 ) return 1;
	return (-1);
}


/*
Description   :  Output 1 Byte Hexadecimal digit to 2Byte ASCII character.  ex) 0x2E --> "2E"
Argument      :   byData - character to output(INPUT)
Return Value  :
Note          :
*/
void PutHTOA(unsigned char byData)
{
	// HIGH DIGIT
	if((byData / 0x10) >= 10)
		PutByte('A'+((byData/0x10)%0x0A));
	else
		PutByte('0'+((byData/0x10)%0x0A));
	// LOW DIGIT
	if((byData % 0x10) >= 10)
		PutByte('A' + ((byData%0x10)%0x0A));
	else
		PutByte('0' + ((byData%0x10)%0x0A));
}

/*
Description   : Output 2 Byte Integer to 4Byte ASCII character ex) 0x12FD --> "12FD"
Argument      :    byData - Integer to output(INPUT)
Return Value  :
Note          :
*/
void PutITOA(unsigned int byData)
{
	PutHTOA(byData / 0x100);
	PutHTOA(byData % 0x100);
}

/*
Description   :   Output 4 Byte Long to 8Byte ASCII character. ex) 0x001234FF --> "001234FF"
Argument      :  byData - Long to output (INPUT)
Return Value  :
Note          :
*/
void PutLTOA(unsigned long byData)
{
	// upper 2 Byte
	PutITOA(byData / 0x10000);
	// lower 2 Byte
	PutITOA(byData % 0x10000);
}

/*
Description   :  Output to Serial.
Argument      :  Str - Character Stream to output (INPUT)
Return Value  :
Note          :  Version 2.0
*/
void PutString(char *Str)
{
	unsigned int i;
	for (i = 0; Str[i] != '\0'; i++)	PutByte(Str[i]);

}

/*
Description   :  Output to Serial and then specific character,'Carrage Return & New Line'.
Argument      : Str -  Character Stream to output(INPUT)
Return Value  :
Note          :  Version 2.0
*/
void PutStringLn(char * Str)
{
    PutString(Str);
	PutByte(0x0a);
	PutByte(0x0d);
}


/*
###############################################################################
Function Prototype Definition Part
###############################################################################
*/



/*
Description   :  Read 1 line string from Serial.
Argument      :  1 line string to be retuned
Return Value  :
Note          :  Version 2.0
		 if First input character is '!' then display previous command and wait enter.
*/
void GetString(char* str)
{
	char c;
	char * tsrc = str;
	char IsFirst = 1;
	while((c=GetByte()) != 0x0D)
	{
		if(IsFirst && c=='!')
		{
			PutString(str);
			while(*str != '\0')str++;
			IsFirst = 0;
			continue;
		}
		if(c == 0x08 && tsrc != str)
		{
	       		PutByte(0x08);
			PutByte(' ');
			PutByte(0x08);
			str--;
			continue;
		}
		else if (c == 0x1B)
		{
			while(tsrc != str)
			{
		       		PutByte(0x08);
				PutByte(' ');
				PutByte(0x08);
				str--;
			}
			IsFirst = 1;
			continue;			
		}
		else if( (c < 32 || c > 126) && c != '\t')	continue;
		PutByte(c);
		*str++ = c;
		IsFirst = 0;
	}
	*str = '\0';
	PutStringLn("");
}

#endif // end __SERIAL_UNUSED

